# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SystemMessage"]


class SystemMessage(BaseModel):
    id: str

    content: str
    """The message content sent by the system"""

    date: datetime

    is_err: Optional[bool] = None

    message_type: Optional[Literal["system_message"]] = None
    """The type of the message."""

    name: Optional[str] = None

    otid: Optional[str] = None

    run_id: Optional[str] = None

    sender_id: Optional[str] = None

    seq_id: Optional[int] = None

    step_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from ..agents.message_type import MessageType
from ..message_create_param import MessageCreateParam
from ..agents.approval_create_param import ApprovalCreateParam

__all__ = ["MessageStreamParams", "Message"]


class MessageStreamParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """The messages to be sent to the agent."""

    assistant_message_tool_kwarg: str
    """The name of the message argument in the designated message tool.

    Still supported for legacy agent types, but deprecated for letta_v1_agent
    onward.
    """

    assistant_message_tool_name: str
    """The name of the designated message tool.

    Still supported for legacy agent types, but deprecated for letta_v1_agent
    onward.
    """

    background: bool
    """Whether to process the request in the background."""

    enable_thinking: str
    """
    If set to True, enables reasoning before responses or tool calls from the agent.
    """

    include_pings: bool
    """
    Whether to include periodic keepalive ping messages in the stream to prevent
    connection timeouts.
    """

    include_return_message_types: Optional[List[MessageType]]
    """Only return specified message types in the response.

    If `None` (default) returns all messages.
    """

    max_steps: int
    """Maximum number of steps the agent should take to process the request."""

    stream_tokens: bool
    """
    Flag to determine if individual tokens should be streamed, rather than streaming
    per step.
    """

    use_assistant_message: bool
    """
    Whether the server should parse specific tool call arguments (default
    `send_message`) as `AssistantMessage` objects. Still supported for legacy agent
    types, but deprecated for letta_v1_agent onward.
    """


Message: TypeAlias = Union[MessageCreateParam, ApprovalCreateParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Optional, cast
from typing_extensions import Literal, overload

import httpx

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from ...types import mcp_server_create_params, mcp_server_modify_params
from .refresh import (
    RefreshResource,
    AsyncRefreshResource,
    RefreshResourceWithRawResponse,
    AsyncRefreshResourceWithRawResponse,
    RefreshResourceWithStreamingResponse,
    AsyncRefreshResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.mcp_server_list_response import McpServerListResponse
from ...types.mcp_server_create_response import McpServerCreateResponse
from ...types.mcp_server_modify_response import McpServerModifyResponse
from ...types.mcp_server_retrieve_response import McpServerRetrieveResponse

__all__ = ["McpServersResource", "AsyncMcpServersResource"]


class McpServersResource(SyncAPIResource):
    @cached_property
    def tools(self) -> ToolsResource:
        return ToolsResource(self._client)

    @cached_property
    def refresh(self) -> RefreshResource:
        return RefreshResource(self._client)

    @cached_property
    def with_raw_response(self) -> McpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return McpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return McpServersResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        args: SequenceNotStr[str],
        command: str,
        server_name: str,
        env: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          args: The arguments to pass to the command

          command: The command to run (MCP 'local' client will run this command)

          server_name: The name of the server

          env: Environment variables to set

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        server_name: str,
        server_url: str,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          server_name: The name of the server

          server_url: The URL of the server

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom HTTP headers to include with requests

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        server_name: str,
        server_url: str,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          server_name: The name of the server

          server_url: The URL of the server

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom HTTP headers to include with requests

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["args", "command", "server_name"], ["server_name", "server_url"])
    def create(
        self,
        *,
        args: SequenceNotStr[str] | Omit = omit,
        command: str | Omit = omit,
        server_name: str,
        env: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        server_url: str | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        return cast(
            McpServerCreateResponse,
            self._post(
                "/v1/mcp-servers/",
                body=maybe_transform(
                    {
                        "args": args,
                        "command": command,
                        "server_name": server_name,
                        "env": env,
                        "type": type,
                        "server_url": server_url,
                        "auth_header": auth_header,
                        "auth_token": auth_token,
                        "custom_headers": custom_headers,
                    },
                    mcp_server_create_params.McpServerCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerRetrieveResponse:
        """
        Get a specific MCP server

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return cast(
            McpServerRetrieveResponse,
            self._get(
                f"/v1/mcp-servers/{mcp_server_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerListResponse:
        """Get a list of all configured MCP servers"""
        return self._get(
            "/v1/mcp-servers/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerListResponse,
        )

    def delete(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an MCP server by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def connect(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Connect to an MCP server with support for OAuth via SSE.

        Returns a stream of
        events handling authorization state and exchange if OAuth is required.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._get(
            f"/v1/mcp-servers/connect/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    @overload
    def modify(
        self,
        mcp_server_id: str,
        *,
        args: Optional[SequenceNotStr[str]] | Omit = omit,
        command: Optional[str] | Omit = omit,
        env: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          args: The arguments to pass to the command

          command: The command to run the MCP server

          env: Environment variables to set

          server_name: The name of the MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def modify(
        self,
        mcp_server_id: str,
        *,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          token: The authentication token (internal)

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom headers to send with requests

          server_name: The name of the MCP server

          server_url: The URL of the SSE MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def modify(
        self,
        mcp_server_id: str,
        *,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          token: The authentication token (internal)

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom headers to send with requests

          server_name: The name of the MCP server

          server_url: The URL of the Streamable HTTP MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    def modify(
        self,
        mcp_server_id: str,
        *,
        args: Optional[SequenceNotStr[str]] | Omit = omit,
        command: Optional[str] | Omit = omit,
        env: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return cast(
            McpServerModifyResponse,
            self._patch(
                f"/v1/mcp-servers/{mcp_server_id}",
                body=maybe_transform(
                    {
                        "args": args,
                        "command": command,
                        "env": env,
                        "server_name": server_name,
                        "token": token,
                        "auth_header": auth_header,
                        "auth_token": auth_token,
                        "custom_headers": custom_headers,
                        "server_url": server_url,
                    },
                    mcp_server_modify_params.McpServerModifyParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerModifyResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncMcpServersResource(AsyncAPIResource):
    @cached_property
    def tools(self) -> AsyncToolsResource:
        return AsyncToolsResource(self._client)

    @cached_property
    def refresh(self) -> AsyncRefreshResource:
        return AsyncRefreshResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMcpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncMcpServersResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        args: SequenceNotStr[str],
        command: str,
        server_name: str,
        env: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          args: The arguments to pass to the command

          command: The command to run (MCP 'local' client will run this command)

          server_name: The name of the server

          env: Environment variables to set

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        server_name: str,
        server_url: str,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          server_name: The name of the server

          server_url: The URL of the server

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom HTTP headers to include with requests

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        server_name: str,
        server_url: str,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Add a new MCP server to the Letta MCP server config

        Args:
          server_name: The name of the server

          server_url: The URL of the server

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom HTTP headers to include with requests

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["args", "command", "server_name"], ["server_name", "server_url"])
    async def create(
        self,
        *,
        args: SequenceNotStr[str] | Omit = omit,
        command: str | Omit = omit,
        server_name: str,
        env: Optional[Dict[str, str]] | Omit = omit,
        type: Literal["sse", "stdio", "streamable_http"] | Omit = omit,
        server_url: str | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        return cast(
            McpServerCreateResponse,
            await self._post(
                "/v1/mcp-servers/",
                body=await async_maybe_transform(
                    {
                        "args": args,
                        "command": command,
                        "server_name": server_name,
                        "env": env,
                        "type": type,
                        "server_url": server_url,
                        "auth_header": auth_header,
                        "auth_token": auth_token,
                        "custom_headers": custom_headers,
                    },
                    mcp_server_create_params.McpServerCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerRetrieveResponse:
        """
        Get a specific MCP server

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return cast(
            McpServerRetrieveResponse,
            await self._get(
                f"/v1/mcp-servers/{mcp_server_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerListResponse:
        """Get a list of all configured MCP servers"""
        return await self._get(
            "/v1/mcp-servers/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerListResponse,
        )

    async def delete(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an MCP server by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def connect(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Connect to an MCP server with support for OAuth via SSE.

        Returns a stream of
        events handling authorization state and exchange if OAuth is required.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._get(
            f"/v1/mcp-servers/connect/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    @overload
    async def modify(
        self,
        mcp_server_id: str,
        *,
        args: Optional[SequenceNotStr[str]] | Omit = omit,
        command: Optional[str] | Omit = omit,
        env: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          args: The arguments to pass to the command

          command: The command to run the MCP server

          env: Environment variables to set

          server_name: The name of the MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def modify(
        self,
        mcp_server_id: str,
        *,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          token: The authentication token (internal)

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom headers to send with requests

          server_name: The name of the MCP server

          server_url: The URL of the SSE MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def modify(
        self,
        mcp_server_id: str,
        *,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        """
        Update an existing MCP server configuration

        Args:
          token: The authentication token (internal)

          auth_header: The name of the authentication header (e.g., 'Authorization')

          auth_token: The authentication token or API key value

          custom_headers: Custom headers to send with requests

          server_name: The name of the MCP server

          server_url: The URL of the Streamable HTTP MCP server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    async def modify(
        self,
        mcp_server_id: str,
        *,
        args: Optional[SequenceNotStr[str]] | Omit = omit,
        command: Optional[str] | Omit = omit,
        env: Optional[Dict[str, str]] | Omit = omit,
        server_name: Optional[str] | Omit = omit,
        token: Optional[str] | Omit = omit,
        auth_header: Optional[str] | Omit = omit,
        auth_token: Optional[str] | Omit = omit,
        custom_headers: Optional[Dict[str, str]] | Omit = omit,
        server_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerModifyResponse:
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return cast(
            McpServerModifyResponse,
            await self._patch(
                f"/v1/mcp-servers/{mcp_server_id}",
                body=await async_maybe_transform(
                    {
                        "args": args,
                        "command": command,
                        "env": env,
                        "server_name": server_name,
                        "token": token,
                        "auth_header": auth_header,
                        "auth_token": auth_token,
                        "custom_headers": custom_headers,
                        "server_url": server_url,
                    },
                    mcp_server_modify_params.McpServerModifyParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, McpServerModifyResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class McpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_raw_response_wrapper(
            mcp_servers.delete,
        )
        self.connect = to_raw_response_wrapper(
            mcp_servers.connect,
        )
        self.modify = to_raw_response_wrapper(
            mcp_servers.modify,
        )

    @cached_property
    def tools(self) -> ToolsResourceWithRawResponse:
        return ToolsResourceWithRawResponse(self._mcp_servers.tools)

    @cached_property
    def refresh(self) -> RefreshResourceWithRawResponse:
        return RefreshResourceWithRawResponse(self._mcp_servers.refresh)


class AsyncMcpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            mcp_servers.delete,
        )
        self.connect = async_to_raw_response_wrapper(
            mcp_servers.connect,
        )
        self.modify = async_to_raw_response_wrapper(
            mcp_servers.modify,
        )

    @cached_property
    def tools(self) -> AsyncToolsResourceWithRawResponse:
        return AsyncToolsResourceWithRawResponse(self._mcp_servers.tools)

    @cached_property
    def refresh(self) -> AsyncRefreshResourceWithRawResponse:
        return AsyncRefreshResourceWithRawResponse(self._mcp_servers.refresh)


class McpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_streamed_response_wrapper(
            mcp_servers.delete,
        )
        self.connect = to_streamed_response_wrapper(
            mcp_servers.connect,
        )
        self.modify = to_streamed_response_wrapper(
            mcp_servers.modify,
        )

    @cached_property
    def tools(self) -> ToolsResourceWithStreamingResponse:
        return ToolsResourceWithStreamingResponse(self._mcp_servers.tools)

    @cached_property
    def refresh(self) -> RefreshResourceWithStreamingResponse:
        return RefreshResourceWithStreamingResponse(self._mcp_servers.refresh)


class AsyncMcpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            mcp_servers.delete,
        )
        self.connect = async_to_streamed_response_wrapper(
            mcp_servers.connect,
        )
        self.modify = async_to_streamed_response_wrapper(
            mcp_servers.modify,
        )

    @cached_property
    def tools(self) -> AsyncToolsResourceWithStreamingResponse:
        return AsyncToolsResourceWithStreamingResponse(self._mcp_servers.tools)

    @cached_property
    def refresh(self) -> AsyncRefreshResourceWithStreamingResponse:
        return AsyncRefreshResourceWithStreamingResponse(self._mcp_servers.refresh)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .tool_return_param import ToolReturnParam

__all__ = ["ApprovalCreateParam", "Approval", "ApprovalApprovalReturn"]


class ApprovalApprovalReturn(TypedDict, total=False):
    approve: Required[bool]
    """Whether the tool has been approved"""

    tool_call_id: Required[str]
    """The ID of the tool call that corresponds to this approval"""

    reason: Optional[str]
    """An optional explanation for the provided approval status"""

    type: Literal["approval"]
    """The message type to be created."""


Approval: TypeAlias = Union[ApprovalApprovalReturn, ToolReturnParam]


class ApprovalCreateParam(TypedDict, total=False):
    approval_request_id: Optional[str]
    """The message ID of the approval request"""

    approvals: Optional[Iterable[Approval]]
    """The list of approval responses"""

    approve: Optional[bool]
    """Whether the tool has been approved"""

    reason: Optional[str]
    """An optional explanation for the provided approval status"""

    type: Literal["approval"]
    """The message type to be created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .llm_config_param import LlmConfigParam
from .init_tool_rule_param import InitToolRuleParam
from .child_tool_rule_param import ChildToolRuleParam
from .embedding_config_param import EmbeddingConfigParam
from .parent_tool_rule_param import ParentToolRuleParam
from .continue_tool_rule_param import ContinueToolRuleParam
from .terminal_tool_rule_param import TerminalToolRuleParam
from .text_response_format_param import TextResponseFormatParam
from .conditional_tool_rule_param import ConditionalToolRuleParam
from .json_object_response_format_param import JsonObjectResponseFormatParam
from .json_schema_response_format_param import JsonSchemaResponseFormatParam
from .requires_approval_tool_rule_param import RequiresApprovalToolRuleParam
from .max_count_per_step_tool_rule_param import MaxCountPerStepToolRuleParam
from .required_before_exit_tool_rule_param import RequiredBeforeExitToolRuleParam

__all__ = ["AgentModifyParams", "ResponseFormat", "ToolRule"]


class AgentModifyParams(TypedDict, total=False):
    base_template_id: Optional[str]
    """The base template id of the agent."""

    block_ids: Optional[SequenceNotStr[str]]
    """The ids of the blocks used by the agent."""

    context_window_limit: Optional[int]
    """The context window limit used by the agent."""

    description: Optional[str]
    """The description of the agent."""

    embedding: Optional[str]
    """
    The embedding configuration handle used by the agent, specified in the format
    provider/model-name.
    """

    embedding_config: Optional[EmbeddingConfigParam]
    """Configuration for embedding model connection and processing parameters."""

    enable_sleeptime: Optional[bool]
    """If set to True, memory management will move to a background agent thread."""

    hidden: Optional[bool]
    """If set to True, the agent will be hidden."""

    identity_ids: Optional[SequenceNotStr[str]]
    """The ids of the identities associated with this agent."""

    last_run_completion: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """The timestamp when the agent last completed a run."""

    last_run_duration_ms: Optional[int]
    """The duration in milliseconds of the agent's last run."""

    llm_config: Optional[LlmConfigParam]
    """Configuration for Language Model (LLM) connection and generation parameters."""

    max_files_open: Optional[int]
    """Maximum number of files that can be open at once for this agent.

    Setting this too high may exceed the context window, which will break the agent.
    """

    max_tokens: Optional[int]
    """The maximum number of tokens to generate, including reasoning step.

    If not set, the model will use its default value.
    """

    message_buffer_autoclear: Optional[bool]
    """
    If set to True, the agent will not remember previous messages (though the agent
    will still retain state via core memory blocks and archival/recall memory). Not
    recommended unless you have an advanced use case.
    """

    message_ids: Optional[SequenceNotStr[str]]
    """The ids of the messages in the agent's in-context memory."""

    metadata: Optional[Dict[str, object]]
    """The metadata of the agent."""

    model: Optional[str]
    """
    The LLM configuration handle used by the agent, specified in the format
    provider/model-name, as an alternative to specifying llm_config.
    """

    name: Optional[str]
    """The name of the agent."""

    parallel_tool_calls: Optional[bool]
    """If set to True, enables parallel tool calling. Defaults to False."""

    per_file_view_window_char_limit: Optional[int]
    """The per-file view window character limit for this agent.

    Setting this too high may exceed the context window, which will break the agent.
    """

    project_id: Optional[str]
    """The id of the project the agent belongs to."""

    reasoning: Optional[bool]
    """Whether to enable reasoning for this agent."""

    response_format: Optional[ResponseFormat]
    """The response format for the agent."""

    secrets: Optional[Dict[str, str]]
    """The environment variables for tool execution specific to this agent."""

    source_ids: Optional[SequenceNotStr[str]]
    """The ids of the sources used by the agent."""

    system: Optional[str]
    """The system prompt used by the agent."""

    tags: Optional[SequenceNotStr[str]]
    """The tags associated with the agent."""

    template_id: Optional[str]
    """The id of the template the agent belongs to."""

    timezone: Optional[str]
    """The timezone of the agent (IANA format)."""

    tool_exec_environment_variables: Optional[Dict[str, str]]
    """Deprecated: use `secrets` field instead"""

    tool_ids: Optional[SequenceNotStr[str]]
    """The ids of the tools used by the agent."""

    tool_rules: Optional[Iterable[ToolRule]]
    """The tool rules governing the agent."""


ResponseFormat: TypeAlias = Union[TextResponseFormatParam, JsonSchemaResponseFormatParam, JsonObjectResponseFormatParam]

ToolRule: TypeAlias = Union[
    ChildToolRuleParam,
    InitToolRuleParam,
    TerminalToolRuleParam,
    ConditionalToolRuleParam,
    ContinueToolRuleParam,
    RequiredBeforeExitToolRuleParam,
    MaxCountPerStepToolRuleParam,
    ParentToolRuleParam,
    RequiresApprovalToolRuleParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .identity import Identity

__all__ = ["IdentityListResponse"]

IdentityListResponse: TypeAlias = List[Identity]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["ToolCountParams"]


class ToolCountParams(TypedDict, total=False):
    exclude_letta_tools: Optional[bool]
    """Exclude built-in Letta tools from the count"""

    exclude_tool_types: Optional[SequenceNotStr[str]]
    """Tool type(s) to exclude - accepts repeated params or comma-separated values"""

    name: Optional[str]

    names: Optional[SequenceNotStr[str]]
    """Filter by specific tool names"""

    return_only_letta_tools: Optional[bool]
    """Count only tools with tool*type starting with 'letta*'"""

    search: Optional[str]
    """Search tool names (case-insensitive partial match)"""

    tool_ids: Optional[SequenceNotStr[str]]
    """Filter by specific tool IDs - accepts repeated params or comma-separated values"""

    tool_types: Optional[SequenceNotStr[str]]
    """Filter by tool type(s) - accepts repeated params or comma-separated values"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .run import Run as Run
from .block import Block as Block
from .message import Message as Message
from .job_type import JobType as JobType
from .tool_call import ToolCall as ToolCall
from .job_status import JobStatus as JobStatus
from .tool_return import ToolReturn as ToolReturn
from .message_role import MessageRole as MessageRole
from .message_type import MessageType as MessageType
from .text_content import TextContent as TextContent
from .user_message import UserMessage as UserMessage
from .image_content import ImageContent as ImageContent
from .letta_response import LettaResponse as LettaResponse
from .system_message import SystemMessage as SystemMessage
from .tool_call_delta import ToolCallDelta as ToolCallDelta
from .file_list_params import FileListParams as FileListParams
from .tool_list_params import ToolListParams as ToolListParams
from .assistant_message import AssistantMessage as AssistantMessage
from .block_list_params import BlockListParams as BlockListParams
from .group_list_params import GroupListParams as GroupListParams
from .reasoning_content import ReasoningContent as ReasoningContent
from .reasoning_message import ReasoningMessage as ReasoningMessage
from .tool_call_content import ToolCallContent as ToolCallContent
from .tool_call_message import ToolCallMessage as ToolCallMessage
from .tool_return_param import ToolReturnParam as ToolReturnParam
from .file_list_response import FileListResponse as FileListResponse
from .file_open_response import FileOpenResponse as FileOpenResponse
from .folder_list_params import FolderListParams as FolderListParams
from .text_content_param import TextContentParam as TextContentParam
from .tool_list_response import ToolListResponse as ToolListResponse
from .block_modify_params import BlockModifyParams as BlockModifyParams
from .image_content_param import ImageContentParam as ImageContentParam
from .letta_message_union import LettaMessageUnion as LettaMessageUnion
from .message_list_params import MessageListParams as MessageListParams
from .message_send_params import MessageSendParams as MessageSendParams
from .tool_return_content import ToolReturnContent as ToolReturnContent
from .folder_list_response import FolderListResponse as FolderListResponse
from .message_reset_params import MessageResetParams as MessageResetParams
from .approval_create_param import ApprovalCreateParam as ApprovalCreateParam
from .message_cancel_params import MessageCancelParams as MessageCancelParams
from .message_modify_params import MessageModifyParams as MessageModifyParams
from .message_stream_params import MessageStreamParams as MessageStreamParams
from .file_close_all_response import FileCloseAllResponse as FileCloseAllResponse
from .message_cancel_response import MessageCancelResponse as MessageCancelResponse
from .message_modify_response import MessageModifyResponse as MessageModifyResponse
from .reasoning_content_param import ReasoningContentParam as ReasoningContentParam
from .tool_call_content_param import ToolCallContentParam as ToolCallContentParam
from .approval_request_message import ApprovalRequestMessage as ApprovalRequestMessage
from .hidden_reasoning_message import HiddenReasoningMessage as HiddenReasoningMessage
from .letta_streaming_response import LettaStreamingResponse as LettaStreamingResponse
from .approval_response_message import ApprovalResponseMessage as ApprovalResponseMessage
from .message_send_async_params import MessageSendAsyncParams as MessageSendAsyncParams
from .omitted_reasoning_content import OmittedReasoningContent as OmittedReasoningContent
from .tool_return_content_param import ToolReturnContentParam as ToolReturnContentParam
from .update_user_message_param import UpdateUserMessageParam as UpdateUserMessageParam
from .redacted_reasoning_content import RedactedReasoningContent as RedactedReasoningContent
from .tool_update_approval_params import ToolUpdateApprovalParams as ToolUpdateApprovalParams
from .update_system_message_param import UpdateSystemMessageParam as UpdateSystemMessageParam
from .update_assistant_message_param import UpdateAssistantMessageParam as UpdateAssistantMessageParam
from .update_reasoning_message_param import UpdateReasoningMessageParam as UpdateReasoningMessageParam
from .omitted_reasoning_content_param import OmittedReasoningContentParam as OmittedReasoningContentParam
from .letta_user_message_content_union import LettaUserMessageContentUnion as LettaUserMessageContentUnion
from .redacted_reasoning_content_param import RedactedReasoningContentParam as RedactedReasoningContentParam
from .letta_assistant_message_content_union import (
    LettaAssistantMessageContentUnion as LettaAssistantMessageContentUnion,
)
from .letta_user_message_content_union_param import (
    LettaUserMessageContentUnionParam as LettaUserMessageContentUnionParam,
)
from .letta_assistant_message_content_union_param import (
    LettaAssistantMessageContentUnionParam as LettaAssistantMessageContentUnionParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from letta_client import Letta, AsyncLetta
from letta_client.pagination import SyncArrayPage, AsyncArrayPage
from letta_client.types.agents import LettaMessageUnion

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Letta) -> None:
        message = client.runs.messages.list(
            run_id="run_id",
        )
        assert_matches_type(SyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Letta) -> None:
        message = client.runs.messages.list(
            run_id="run_id",
            after="after",
            before="before",
            limit=0,
            order="asc",
            order_by="created_at",
        )
        assert_matches_type(SyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Letta) -> None:
        response = client.runs.messages.with_raw_response.list(
            run_id="run_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(SyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Letta) -> None:
        with client.runs.messages.with_streaming_response.list(
            run_id="run_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(SyncArrayPage[LettaMessageUnion], message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.runs.messages.with_raw_response.list(
                run_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stream(self, client: Letta) -> None:
        message_stream = client.runs.messages.stream(
            run_id="run_id",
        )
        message_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stream_with_all_params(self, client: Letta) -> None:
        message_stream = client.runs.messages.stream(
            run_id="run_id",
            batch_size=0,
            include_pings=True,
            poll_interval=0,
            starting_after=0,
        )
        message_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_stream(self, client: Letta) -> None:
        response = client.runs.messages.with_raw_response.stream(
            run_id="run_id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_stream(self, client: Letta) -> None:
        with client.runs.messages.with_streaming_response.stream(
            run_id="run_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_stream(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.runs.messages.with_raw_response.stream(
                run_id="",
            )


class TestAsyncMessages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLetta) -> None:
        message = await async_client.runs.messages.list(
            run_id="run_id",
        )
        assert_matches_type(AsyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLetta) -> None:
        message = await async_client.runs.messages.list(
            run_id="run_id",
            after="after",
            before="before",
            limit=0,
            order="asc",
            order_by="created_at",
        )
        assert_matches_type(AsyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLetta) -> None:
        response = await async_client.runs.messages.with_raw_response.list(
            run_id="run_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(AsyncArrayPage[LettaMessageUnion], message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLetta) -> None:
        async with async_client.runs.messages.with_streaming_response.list(
            run_id="run_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(AsyncArrayPage[LettaMessageUnion], message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.runs.messages.with_raw_response.list(
                run_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stream(self, async_client: AsyncLetta) -> None:
        message_stream = await async_client.runs.messages.stream(
            run_id="run_id",
        )
        await message_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stream_with_all_params(self, async_client: AsyncLetta) -> None:
        message_stream = await async_client.runs.messages.stream(
            run_id="run_id",
            batch_size=0,
            include_pings=True,
            poll_interval=0,
            starting_after=0,
        )
        await message_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_stream(self, async_client: AsyncLetta) -> None:
        response = await async_client.runs.messages.with_raw_response.stream(
            run_id="run_id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_stream(self, async_client: AsyncLetta) -> None:
        async with async_client.runs.messages.with_streaming_response.stream(
            run_id="run_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_stream(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.runs.messages.with_raw_response.stream(
                run_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)

__all__ = [
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.mcp_servers import refresh_trigger_params

__all__ = ["RefreshResource", "AsyncRefreshResource"]


class RefreshResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RefreshResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return RefreshResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RefreshResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return RefreshResourceWithStreamingResponse(self)

    def trigger(
        self,
        mcp_server_id: str,
        *,
        agent_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Refresh tools for an MCP server by:

        1.

        Fetching current tools from the MCP server
        2. Deleting tools that no longer exist on the server
        3. Updating schemas for existing tools
        4. Adding new tools from the server

        Returns a summary of changes made.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._patch(
            f"/v1/mcp-servers/{mcp_server_id}/refresh",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"agent_id": agent_id}, refresh_trigger_params.RefreshTriggerParams),
            ),
            cast_to=object,
        )


class AsyncRefreshResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRefreshResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRefreshResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRefreshResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncRefreshResourceWithStreamingResponse(self)

    async def trigger(
        self,
        mcp_server_id: str,
        *,
        agent_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Refresh tools for an MCP server by:

        1.

        Fetching current tools from the MCP server
        2. Deleting tools that no longer exist on the server
        3. Updating schemas for existing tools
        4. Adding new tools from the server

        Returns a summary of changes made.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._patch(
            f"/v1/mcp-servers/{mcp_server_id}/refresh",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"agent_id": agent_id}, refresh_trigger_params.RefreshTriggerParams),
            ),
            cast_to=object,
        )


class RefreshResourceWithRawResponse:
    def __init__(self, refresh: RefreshResource) -> None:
        self._refresh = refresh

        self.trigger = to_raw_response_wrapper(
            refresh.trigger,
        )


class AsyncRefreshResourceWithRawResponse:
    def __init__(self, refresh: AsyncRefreshResource) -> None:
        self._refresh = refresh

        self.trigger = async_to_raw_response_wrapper(
            refresh.trigger,
        )


class RefreshResourceWithStreamingResponse:
    def __init__(self, refresh: RefreshResource) -> None:
        self._refresh = refresh

        self.trigger = to_streamed_response_wrapper(
            refresh.trigger,
        )


class AsyncRefreshResourceWithStreamingResponse:
    def __init__(self, refresh: AsyncRefreshResource) -> None:
        self._refresh = refresh

        self.trigger = async_to_streamed_response_wrapper(
            refresh.trigger,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..stop_reason_type import StopReasonType
from .letta_message_union import LettaMessageUnion

__all__ = ["LettaResponse", "StopReason", "Usage"]


class StopReason(BaseModel):
    stop_reason: StopReasonType
    """The reason why execution stopped."""

    message_type: Optional[Literal["stop_reason"]] = None
    """The type of the message."""


class Usage(BaseModel):
    completion_tokens: Optional[int] = None
    """The number of tokens generated by the agent."""

    message_type: Optional[Literal["usage_statistics"]] = None

    prompt_tokens: Optional[int] = None
    """The number of tokens in the prompt."""

    run_ids: Optional[List[str]] = None
    """The background task run IDs associated with the agent interaction"""

    step_count: Optional[int] = None
    """The number of steps taken by the agent."""

    total_tokens: Optional[int] = None
    """The total number of tokens processed by the agent."""


class LettaResponse(BaseModel):
    messages: List[LettaMessageUnion]
    """The messages returned by the agent."""

    stop_reason: StopReason
    """The stop reason from Letta indicating why agent loop stopped execution."""

    usage: Usage
    """The usage statistics of the agent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .step import Step as Step
from .tool import Tool as Tool
from .group import Group as Group
from .folder import Folder as Folder
from .archive import Archive as Archive
from .identity import Identity as Identity
from .batch_job import BatchJob as BatchJob
from .tool_type import ToolType as ToolType
from .agent_type import AgentType as AgentType
from .llm_config import LlmConfig as LlmConfig
from .agent_state import AgentState as AgentState
from .manager_type import ManagerType as ManagerType
from .identity_type import IdentityType as IdentityType
from .provider_type import ProviderType as ProviderType
from .block_response import BlockResponse as BlockResponse
from .init_tool_rule import InitToolRule as InitToolRule
from .provider_trace import ProviderTrace as ProviderTrace
from .child_tool_rule import ChildToolRule as ChildToolRule
from .health_response import HealthResponse as HealthResponse
from .npm_requirement import NpmRequirement as NpmRequirement
from .pip_requirement import PipRequirement as PipRequirement
from .run_list_params import RunListParams as RunListParams
from .tag_list_params import TagListParams as TagListParams
from .embedding_config import EmbeddingConfig as EmbeddingConfig
from .llm_config_param import LlmConfigParam as LlmConfigParam
from .parent_tool_rule import ParentToolRule as ParentToolRule
from .step_list_params import StepListParams as StepListParams
from .stop_reason_type import StopReasonType as StopReasonType
from .tool_list_params import ToolListParams as ToolListParams
from .agent_list_params import AgentListParams as AgentListParams
from .batch_list_params import BatchListParams as BatchListParams
from .block_list_params import BlockListParams as BlockListParams
from .group_list_params import GroupListParams as GroupListParams
from .identity_property import IdentityProperty as IdentityProperty
from .model_list_params import ModelListParams as ModelListParams
from .provider_category import ProviderCategory as ProviderCategory
from .run_list_response import RunListResponse as RunListResponse
from .tag_list_response import TagListResponse as TagListResponse
from .tool_count_params import ToolCountParams as ToolCountParams
from .continue_tool_rule import ContinueToolRule as ContinueToolRule
from .create_block_param import CreateBlockParam as CreateBlockParam
from .folder_list_params import FolderListParams as FolderListParams
from .terminal_tool_rule import TerminalToolRule as TerminalToolRule
from .tool_create_params import ToolCreateParams as ToolCreateParams
from .tool_list_response import ToolListResponse as ToolListResponse
from .tool_modify_params import ToolModifyParams as ToolModifyParams
from .tool_upsert_params import ToolUpsertParams as ToolUpsertParams
from .vector_db_provider import VectorDBProvider as VectorDBProvider
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_modify_params import AgentModifyParams as AgentModifyParams
from .archive_list_params import ArchiveListParams as ArchiveListParams
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .batch_list_response import BatchListResponse as BatchListResponse
from .block_create_params import BlockCreateParams as BlockCreateParams
from .block_modify_params import BlockModifyParams as BlockModifyParams
from .group_create_params import GroupCreateParams as GroupCreateParams
from .group_modify_params import GroupModifyParams as GroupModifyParams
from .model_list_response import ModelListResponse as ModelListResponse
from .tool_count_response import ToolCountResponse as ToolCountResponse
from .tool_return_message import ToolReturnMessage as ToolReturnMessage
from .agent_count_response import AgentCountResponse as AgentCountResponse
from .block_count_response import BlockCountResponse as BlockCountResponse
from .folder_create_params import FolderCreateParams as FolderCreateParams
from .folder_list_response import FolderListResponse as FolderListResponse
from .folder_modify_params import FolderModifyParams as FolderModifyParams
from .group_count_response import GroupCountResponse as GroupCountResponse
from .identity_list_params import IdentityListParams as IdentityListParams
from .init_tool_rule_param import InitToolRuleParam as InitToolRuleParam
from .message_create_param import MessageCreateParam as MessageCreateParam
from .text_response_format import TextResponseFormat as TextResponseFormat
from .agent_retrieve_params import AgentRetrieveParams as AgentRetrieveParams
from .archive_create_params import ArchiveCreateParams as ArchiveCreateParams
from .archive_list_response import ArchiveListResponse as ArchiveListResponse
from .archive_modify_params import ArchiveModifyParams as ArchiveModifyParams
from .child_tool_rule_param import ChildToolRuleParam as ChildToolRuleParam
from .conditional_tool_rule import ConditionalToolRule as ConditionalToolRule
from .dynamic_manager_param import DynamicManagerParam as DynamicManagerParam
from .folder_count_response import FolderCountResponse as FolderCountResponse
from .npm_requirement_param import NpmRequirementParam as NpmRequirementParam
from .pip_requirement_param import PipRequirementParam as PipRequirementParam
from .embedding_config_param import EmbeddingConfigParam as EmbeddingConfigParam
from .identity_create_params import IdentityCreateParams as IdentityCreateParams
from .identity_list_response import IdentityListResponse as IdentityListResponse
from .identity_modify_params import IdentityModifyParams as IdentityModifyParams
from .identity_upsert_params import IdentityUpsertParams as IdentityUpsertParams
from .parent_tool_rule_param import ParentToolRuleParam as ParentToolRuleParam
from .identity_count_response import IdentityCountResponse as IdentityCountResponse
from .identity_property_param import IdentityPropertyParam as IdentityPropertyParam
from .sleeptime_manager_param import SleeptimeManagerParam as SleeptimeManagerParam
from .agent_export_file_params import AgentExportFileParams as AgentExportFileParams
from .agent_import_file_params import AgentImportFileParams as AgentImportFileParams
from .continue_tool_rule_param import ContinueToolRuleParam as ContinueToolRuleParam
from .mcp_server_create_params import McpServerCreateParams as McpServerCreateParams
from .mcp_server_list_response import McpServerListResponse as McpServerListResponse
from .mcp_server_modify_params import McpServerModifyParams as McpServerModifyParams
from .supervisor_manager_param import SupervisorManagerParam as SupervisorManagerParam
from .terminal_tool_rule_param import TerminalToolRuleParam as TerminalToolRuleParam
from .round_robin_manager_param import RoundRobinManagerParam as RoundRobinManagerParam
from .agent_environment_variable import AgentEnvironmentVariable as AgentEnvironmentVariable
from .agent_export_file_response import AgentExportFileResponse as AgentExportFileResponse
from .agent_import_file_response import AgentImportFileResponse as AgentImportFileResponse
from .mcp_server_create_response import McpServerCreateResponse as McpServerCreateResponse
from .mcp_server_modify_response import McpServerModifyResponse as McpServerModifyResponse
from .text_response_format_param import TextResponseFormatParam as TextResponseFormatParam
from .conditional_tool_rule_param import ConditionalToolRuleParam as ConditionalToolRuleParam
from .json_object_response_format import JsonObjectResponseFormat as JsonObjectResponseFormat
from .json_schema_response_format import JsonSchemaResponseFormat as JsonSchemaResponseFormat
from .requires_approval_tool_rule import RequiresApprovalToolRule as RequiresApprovalToolRule
from .max_count_per_step_tool_rule import MaxCountPerStepToolRule as MaxCountPerStepToolRule
from .mcp_server_retrieve_response import McpServerRetrieveResponse as McpServerRetrieveResponse
from .voice_sleeptime_manager_param import VoiceSleeptimeManagerParam as VoiceSleeptimeManagerParam
from .required_before_exit_tool_rule import RequiredBeforeExitToolRule as RequiredBeforeExitToolRule
from .tool_upsert_base_tools_response import ToolUpsertBaseToolsResponse as ToolUpsertBaseToolsResponse
from .json_object_response_format_param import JsonObjectResponseFormatParam as JsonObjectResponseFormatParam
from .json_schema_response_format_param import JsonSchemaResponseFormatParam as JsonSchemaResponseFormatParam
from .letta_message_content_union_param import LettaMessageContentUnionParam as LettaMessageContentUnionParam
from .requires_approval_tool_rule_param import RequiresApprovalToolRuleParam as RequiresApprovalToolRuleParam
from .max_count_per_step_tool_rule_param import MaxCountPerStepToolRuleParam as MaxCountPerStepToolRuleParam
from .required_before_exit_tool_rule_param import RequiredBeforeExitToolRuleParam as RequiredBeforeExitToolRuleParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from ..agents.message import Message

__all__ = ["MessageListResponse"]


class MessageListResponse(BaseModel):
    messages: List[Message]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "McpServerListResponse",
    "McpServerListResponseItem",
    "McpServerListResponseItemStdioMcpServer",
    "McpServerListResponseItemSsemcpServer",
    "McpServerListResponseItemStreamableHttpmcpServer",
]


class McpServerListResponseItemStdioMcpServer(BaseModel):
    args: List[str]
    """The arguments to pass to the command"""

    command: str
    """The command to run (MCP 'local' client will run this command)"""

    server_name: str
    """The name of the server"""

    id: Optional[str] = None
    """The human-friendly ID of the Mcp_server"""

    env: Optional[Dict[str, str]] = None
    """Environment variables to set"""

    type: Optional[Literal["sse", "stdio", "streamable_http"]] = None


class McpServerListResponseItemSsemcpServer(BaseModel):
    server_name: str
    """The name of the server"""

    server_url: str
    """The URL of the server"""

    id: Optional[str] = None
    """The human-friendly ID of the Mcp_server"""

    auth_header: Optional[str] = None
    """The name of the authentication header (e.g., 'Authorization')"""

    auth_token: Optional[str] = None
    """The authentication token or API key value"""

    custom_headers: Optional[Dict[str, str]] = None
    """Custom HTTP headers to include with requests"""

    type: Optional[Literal["sse", "stdio", "streamable_http"]] = None


class McpServerListResponseItemStreamableHttpmcpServer(BaseModel):
    server_name: str
    """The name of the server"""

    server_url: str
    """The URL of the server"""

    id: Optional[str] = None
    """The human-friendly ID of the Mcp_server"""

    auth_header: Optional[str] = None
    """The name of the authentication header (e.g., 'Authorization')"""

    auth_token: Optional[str] = None
    """The authentication token or API key value"""

    custom_headers: Optional[Dict[str, str]] = None
    """Custom HTTP headers to include with requests"""

    type: Optional[Literal["sse", "stdio", "streamable_http"]] = None


McpServerListResponseItem: TypeAlias = Union[
    McpServerListResponseItemStdioMcpServer,
    McpServerListResponseItemSsemcpServer,
    McpServerListResponseItemStreamableHttpmcpServer,
]

McpServerListResponse: TypeAlias = List[McpServerListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .archive import Archive

__all__ = ["ArchiveListResponse"]

ArchiveListResponse: TypeAlias = List[Archive]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..agent_state import AgentState

__all__ = ["ToolRunResponse"]


class ToolRunResponse(BaseModel):
    status: Literal["success", "error"]
    """The status of the tool execution and return object"""

    agent_state: Optional[AgentState] = None
    """Representation of an agent's state.

    This is the state of the agent at a given time, and is persisted in the DB
    backend. The state has all the information needed to recreate a persisted agent.

    Parameters: id (str): The unique identifier of the agent. name (str): The name
    of the agent (must be unique to the user). created_at (datetime): The datetime
    the agent was created. message_ids (List[str]): The ids of the messages in the
    agent's in-context memory. memory (Memory): The in-context memory of the agent.
    tools (List[str]): The tools used by the agent. This includes any memory editing
    functions specified in `memory`. system (str): The system prompt used by the
    agent. llm_config (LLMConfig): The LLM configuration used by the agent.
    embedding_config (EmbeddingConfig): The embedding configuration used by the
    agent.
    """

    func_return: Optional[object] = None
    """The function return object"""

    sandbox_config_fingerprint: Optional[str] = None
    """The fingerprint of the config for the sandbox"""

    stderr: Optional[List[str]] = None
    """Captured stderr from the function invocation"""

    stdout: Optional[List[str]] = None
    """Captured stdout (prints, logs) from function invocation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from ...types import ProviderType, model_list_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.provider_type import ProviderType
from ...types.provider_category import ProviderCategory
from ...types.model_list_response import ModelListResponse

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        return EmbeddingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        provider_category: Optional[List[ProviderCategory]] | Omit = omit,
        provider_name: Optional[str] | Omit = omit,
        provider_type: Optional[ProviderType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelListResponse:
        """
        List available LLM models using the asynchronous implementation for improved
        performance

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/models/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "provider_category": provider_category,
                        "provider_name": provider_name,
                        "provider_type": provider_type,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            cast_to=ModelListResponse,
        )


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        return AsyncEmbeddingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        provider_category: Optional[List[ProviderCategory]] | Omit = omit,
        provider_name: Optional[str] | Omit = omit,
        provider_type: Optional[ProviderType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelListResponse:
        """
        List available LLM models using the asynchronous implementation for improved
        performance

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/models/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "provider_category": provider_category,
                        "provider_name": provider_name,
                        "provider_type": provider_type,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            cast_to=ModelListResponse,
        )


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_raw_response_wrapper(
            models.list,
        )

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithRawResponse:
        return EmbeddingsResourceWithRawResponse(self._models.embeddings)


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_raw_response_wrapper(
            models.list,
        )

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithRawResponse:
        return AsyncEmbeddingsResourceWithRawResponse(self._models.embeddings)


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_streamed_response_wrapper(
            models.list,
        )

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithStreamingResponse:
        return EmbeddingsResourceWithStreamingResponse(self._models.embeddings)


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_streamed_response_wrapper(
            models.list,
        )

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        return AsyncEmbeddingsResourceWithStreamingResponse(self._models.embeddings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .message_list_params import MessageListParams as MessageListParams
from .message_send_params import MessageSendParams as MessageSendParams
from .message_modify_params import MessageModifyParams as MessageModifyParams
from .message_stream_params import MessageStreamParams as MessageStreamParams
from .message_modify_response import MessageModifyResponse as MessageModifyResponse
